@@ -106,6 +106,8 @@ def train_from_folder(
     dataset_aug_prob = 0.,
     multi_gpus = False,
     calculate_fid_every = None,
+    calculate_fid_num_images = 12800,
+    clear_fid_cache = False,
     seed = 42,
     amp = False
 ):
@@ -134,6 +136,8 @@ def train_from_folder(
         aug_types = cast_list(aug_types),
         dataset_aug_prob = dataset_aug_prob,
         calculate_fid_every = calculate_fid_every,
+        calculate_fid_num_images = calculate_fid_num_images,
+        clear_fid_cache = clear_fid_cache,
         amp = amp
     )
 
